//Linear servo with optical sensor disc v 1.4:
//Optical sensor 1 to Analog 0
//Optical sensor 2 to Analog 1
//H bridge direction pin to Digital 2
//H bridge PWM signal to Digital 3
//LEDs at digital pins 4 - 13 are optional (indicating state of optical sensors)
//
//For the first test run, set MAX_DUTYCYCLE to 75 in order to lower the maximum torque in case something goes wrong.
//When connecting the motor to the microcontroller for the first time, you have to consider the rotational direction!
//When turning the motor by hand, the microcontroller must switch the motor in such a way, that it turns contraty to your manual movement.
//If not, swap the polarity of the motor.
//
//Code to run on the command computer see:
//commands-from-computer.c
//
//Source & info: www.HomoFaciens.de/technics-computer-arduino-uno_en_navion.htm


//#include <avr/io.h>
//#include <util/delay.h>
#include <Servo.h> 

int sensor_00 = 0;
int sensor_01 = 0;
int dutyCycle = 100; // 10 - 255
int setValue = 100;
int setValueOld = 100;
int readValue = 100;
int readValueOld = 100;
int step = 0;
int timeDiffI = 0;
int timeDiffS = 0;
unsigned long millisStart = 0;
int moveDirection = 0;


//Change values below to adapt your motor
//Set MAX_DUTYCYCLE to 75 for the first test run!

#define P_FRACTION 2.0        //0.0 - 10.0 (2.0)
#define I_FRACTION 2.0        //0.0 - 10.0 (2.0)
#define D_FRACTION 0.0        //0.0 - ??? (0.0) not been tested yet...
#define V_WINDOW 2            //10 - 1000 (2)
#define MIN_DUTYCYCLE 25      //0 - 255 (25)
#define MAX_DUTYCYCLE 255     //0 - 255 (255)
#define SOFT_START 0.05       //0.0 - 1.0 (0.05) 1.00 = OFF
#define EMERGENCY_SHUTDOWN 4  //0 - 100 (4), 0 - OFF
#define SHOOT_THROUGH_PAUSE 10 //Prevent H bridge from shoot through whenever the direction pin is changed

#define SENSOR_PIN_1 A4       //Sensor input 1
#define SENSOR_PIN_2 A5       //Sensor input 2
#define MOTOR_DIRECTION 2     //Direction output motor
#define MOTOR_PWM 3           //PWM output motor
#define LED_01 4
#define LED_02 5
#define LED_03 6
#define LED_04 7
#define LED_05 8
#define LED_06 9
#define LED_07 10
#define LED_08 11
#define LED_09 12
#define LED_10 13

void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('A');   // send a capital A
    delay(300);
  }
}

void setup(){ 
  
  pinMode(MOTOR_DIRECTION, OUTPUT);     
  pinMode(MOTOR_PWM, OUTPUT);     
  
  pinMode(SENSOR_PIN_1, INPUT);
  pinMode(SENSOR_PIN_2, INPUT);
  
  //LED bar (optional)
  pinMode(LED_01, OUTPUT);     
  pinMode(LED_02, OUTPUT);     
  pinMode(LED_03, OUTPUT);     
  pinMode(LED_04, OUTPUT);     
  pinMode(LED_05, OUTPUT);     
  pinMode(LED_06, OUTPUT);     
  pinMode(LED_07, OUTPUT);     
  pinMode(LED_08, OUTPUT);     
  pinMode(LED_09, OUTPUT);     
  pinMode(LED_10, OUTPUT);     
    
  moveDirection = 0;

  // start serial port at 9600 bps:
  Serial.begin(9600);
  establishContact();  // send a byte to establish contact until receiver responds 
}


void loop(){  
  
  sensor_01 = digitalRead(SENSOR_PIN_1);
  sensor_00 = digitalRead(SENSOR_PIN_2);
  
//  PORTB |= 0b00001010;
  
  millisStart = millis();
  while(millis() - millisStart < 15){
  
    if(sensor_00 == 0 && sensor_01 == 0){
      if(step == 1){
        readValue--;
      }
      if(step == 3){
        readValue++;
      }
      step = 0;
      //Set LEDs (optional)
      digitalWrite(LED_01, 1);
      digitalWrite(LED_02, 0);
      digitalWrite(LED_03, 0);
      digitalWrite(LED_04, 0);
    }
  
    if(sensor_00 == 0 && sensor_01 == 1){
      if(step == 2){
        readValue--;
      }
      if(step == 0){
        readValue++;
      }
      step = 1;
      //Set LEDs (optional)
      digitalWrite(LED_01, 0);
      digitalWrite(LED_02, 1);
      digitalWrite(LED_03, 0);
      digitalWrite(LED_04, 0);
    }
  
    if(sensor_00 == 1 && sensor_01 == 1){
      if(step == 3){
        readValue--;
      }
      if(step == 1){
        readValue++;
      }
      step = 2;
      //Set LEDs (optional)
      digitalWrite(LED_01, 0);
      digitalWrite(LED_02, 0);
      digitalWrite(LED_03, 1);
      digitalWrite(LED_04, 0);
    }
  
    if(sensor_00 == 1 && sensor_01 == 0){
      if(step == 0){
        readValue--;
      }
      if(step == 2){
        readValue++;
      }
      step = 3;
      //Set LEDs (optional)
      digitalWrite(LED_01, 0);
      digitalWrite(LED_02, 0);
      digitalWrite(LED_03, 0);
      digitalWrite(LED_04, 1);
    }

    if(abs(readValue - setValue) < V_WINDOW){
      analogWrite(MOTOR_PWM, 0);
      digitalWrite(MOTOR_DIRECTION, 0);
    }
  }


  dutyCycle = abs(readValue - setValue) * P_FRACTION;
  dutyCycle += timeDiffI * I_FRACTION;
  //D_FRACTION has not been tested with the linear servo (yet)
  dutyCycle += D_FRACTION * abs(setValue - setValueOld);

  if(SOFT_START * timeDiffS < 1){
    dutyCycle = dutyCycle * (SOFT_START * timeDiffS);
  }


  timeDiffI++;
  timeDiffS++;
  
  if(dutyCycle < MIN_DUTYCYCLE){
    dutyCycle = MIN_DUTYCYCLE;
  }
  
  if(dutyCycle > MAX_DUTYCYCLE){
    dutyCycle = MAX_DUTYCYCLE;
  }
    
  if(abs(readValue - setValue) < V_WINDOW){
    analogWrite(MOTOR_PWM, 0);
    delayMicroseconds(SHOOT_THROUGH_PAUSE);
    digitalWrite(MOTOR_DIRECTION, 0);

//    dutyCycle = 0;
    timeDiffI = 0;
    timeDiffS = 0;

    if (Serial.available() > 0){//if we get a valid byte, read analog ins:
        setValueOld = setValue;
        //get incoming byte:
        setValue = Serial.read();
        if(setValue == 255){
          if(moveDirection == 0){
            moveDirection = 1;
          }
          else{
            moveDirection = 0;
          }
          setValue = 0;
          readValue = 0;
        }
        else{ 
          if(moveDirection == 0){
            setValue = 25500 - setValue;
            readValue = 25500;
          }
          else{
            setValue = setValue;
            readValue = 0;
          }
        }
        
        //Serial.write(inByte);//Echo received command
    }
  }
  else{
    if(readValue < setValue){
      analogWrite(MOTOR_PWM, 0);
      delayMicroseconds(SHOOT_THROUGH_PAUSE);
      digitalWrite(MOTOR_DIRECTION, 0);
      delayMicroseconds(SHOOT_THROUGH_PAUSE);
      analogWrite(MOTOR_PWM, dutyCycle);
    }
  
    if(readValue > setValue){
      analogWrite(MOTOR_PWM, 0);
      delayMicroseconds(SHOOT_THROUGH_PAUSE);
      digitalWrite(MOTOR_DIRECTION, 1);
      delayMicroseconds(SHOOT_THROUGH_PAUSE);
      analogWrite(MOTOR_PWM, 255 - dutyCycle);
    }
  }
  
  if(readValueOld != readValue && timeDiffI > 2){
    timeDiffI -= 2;
  }
  
  readValueOld = readValue;

}

